/**
 * 注册、登录、注销
 * */
import util from '../../../../libs/util';
import router from '../../../../router';
import userApi from '../../../../api/user';
import swal from 'sweetalert2';

export default {
  namespaced: true,
  actions: {
    /**
     * @description 登录
     * @param {Object} context
     * @param username {String} 用户账号
     * @param password {String} 密码
     */
    async login({ dispatch }, { username = '', password = '' } = {}) {
      const res = await userApi.login({ account: username, password });
      dispatch('admin/account/saveLoginData', res, { root: true });
    },
    /**
     * @description 保存登录信息
     * @param {Object} context
     * @param user {User} 用户login
     * @param token {String} 令牌
     * @param data {String} 数据
     */
    async saveLoginData({ dispatch }, { user, token, data } = {}) {
      // 设置 cookie 一定要存 uuid 和 token 两个 cookie
      // 整个系统依赖这两个数据进行校验和存储
      // uuid 是用户身份唯一标识 用户注册的时候确定 并且不可改变 不可重复
      // token 代表用户当前登录状态 建议在网络请求中携带 token
      // 如有必要 token 需要定时更新，默认保存一天，可在 setting.js 中修改
      // 如果你的 token 不是通过 cookie 携带，而是普通字段，也可视情况存储在 localStorage
      util.cookies.set('uuid', user.id);
      util.cookies.set('token', token);
      // 设置 vuex 用户信息
      await dispatch('admin/user/set', user, { root: true });
      await dispatch('admin/user/setData', data, { root: true });
      // 用户登录后从持久化数据加载一系列的设置
      await dispatch('load');
      // 获取动态菜单
      await dispatch('admin/menu/getMenuList', true, { root: true });
    },
    /**
     * @description 退出登录
     * */
    logout({ dispatch }, { confirm = false } = {}) {
      async function logout() {
        // 删除cookie
        util.cookies.remove('token');
        util.cookies.remove('uuid');
        // 清空 vuex 用户信息
        await dispatch('admin/user/set', {}, { root: true });
        // 跳转路由
        if (router.currentRoute.value.path != '/login') {
          router.push('/login?redirect=' + router.options.history.location);
        }
      }

      if (confirm) {
        swal
          .fire({
            title: window.$app.$t('Front_Msg_Sure_To_Logout'),
            icon: 'question',
            showCancelButton: true,
            confirmButtonText: window.$app.$t('Front_Btn_OK'),
            cancelButtonText: window.$app.$t('Front_Btn_Cancel')
          })
          .then(result => {
            if (result.value) {
              logout();
            }
          });
      } else {
        logout();
      }
    },
    /**
     * @description 身份认证失败
     * */
    authFailed({ dispatch }) {
      dispatch('logout');
    },
    /**
     * @description 用户登录后从持久化数据加载一系列的设置
     * @param {Object} state vuex state
     * @param {Object} dispatch vuex dispatch
     * @param {Object} loadOpenedTabs 是否加载页签信息
     */
    async load({ dispatch }, { loadOpenedTabs = true } = {}) {
      // 加载用户登录信息
      await dispatch('admin/user/load', null, { root: true });
      // 持久化数据加载上次退出时的多页列表
      //await dispatch('admin/page/openedLoad', { loadOpenedTabs }, { root: true });
    }
  }
};
